@@ -27,8 +27,6 @@
  *  \ingroup bke
  */
 
-
-
 #include <math.h>
 #include <stdio.h>
 #include <stddef.h>
@@ -39,7 +37,7 @@
 
 #include "DNA_anim_types.h"
 
-#include "BLF_translation.h"
+#include "BLT_translation.h"
 
 #include "BLI_blenlib.h"
 #include "BLI_ghash.h"
@@ -1042,7 +1040,7 @@ static void fmods_init_typeinfo(void)
 /* This function should be used for getting the appropriate type-info when only
  * a F-Curve modifier type is known
  */
-FModifierTypeInfo *get_fmodifier_typeinfo(int type)
+const FModifierTypeInfo *get_fmodifier_typeinfo(int type)
 {
 	/* initialize the type-info list? */
 	if (FMI_INIT) {
@@ -1067,7 +1065,7 @@ FModifierTypeInfo *get_fmodifier_typeinfo(int type)
 /* This function should always be used to get the appropriate type-info, as it
  * has checks which prevent segfaults in some weird cases.
  */
-FModifierTypeInfo *fmodifier_get_typeinfo(FModifier *fcm)
+const FModifierTypeInfo *fmodifier_get_typeinfo(FModifier *fcm)
 {
 	/* only return typeinfo for valid modifiers */
 	if (fcm)
@@ -1081,7 +1079,7 @@ FModifierTypeInfo *fmodifier_get_typeinfo(FModifier *fcm)
 /* Add a new F-Curve Modifier to the given F-Curve of a certain type */
 FModifier *add_fmodifier(ListBase *modifiers, int type)
 {
-	FModifierTypeInfo *fmi = get_fmodifier_typeinfo(type);
+	const FModifierTypeInfo *fmi = get_fmodifier_typeinfo(type);
 	FModifier *fcm;
 	
 	/* sanity checks */
@@ -1121,7 +1119,7 @@ FModifier *add_fmodifier(ListBase *modifiers, int type)
 /* Make a copy of the specified F-Modifier */
 FModifier *copy_fmodifier(FModifier *src)
 {
-	FModifierTypeInfo *fmi = fmodifier_get_typeinfo(src);
+	const FModifierTypeInfo *fmi = fmodifier_get_typeinfo(src);
 	FModifier *dst;
 	
 	/* sanity check */
@@ -1155,7 +1153,7 @@ void copy_fmodifiers(ListBase *dst, ListBase *src)
 	BLI_duplicatelist(dst, src);
 	
 	for (fcm = dst->first, srcfcm = src->first; fcm && srcfcm; srcfcm = srcfcm->next, fcm = fcm->next) {
-		FModifierTypeInfo *fmi = fmodifier_get_typeinfo(fcm);
+		const FModifierTypeInfo *fmi = fmodifier_get_typeinfo(fcm);
 		
 		/* make a new copy of the F-Modifier's data */
 		fcm->data = MEM_dupallocN(fcm->data);
@@ -1169,11 +1167,11 @@ void copy_fmodifiers(ListBase *dst, ListBase *src)
 /* Remove and free the given F-Modifier from the given stack  */
 bool remove_fmodifier(ListBase *modifiers, FModifier *fcm)
 {
-	FModifierTypeInfo *fmi = fmodifier_get_typeinfo(fcm);
+	const FModifierTypeInfo *fmi = fmodifier_get_typeinfo(fcm);
 	
 	/* sanity check */
 	if (fcm == NULL)
-		return 0;
+		return false;
 	
 	/* free modifier's special data (stored inside fcm->data) */
 	if (fcm->data) {
@@ -1187,13 +1185,13 @@ bool remove_fmodifier(ListBase *modifiers, FModifier *fcm)
 	/* remove modifier from stack */
 	if (modifiers) {
 		BLI_freelinkN(modifiers, fcm);
-		return 1;
+		return true;
 	}
 	else {
 		/* XXX this case can probably be removed some day, as it shouldn't happen... */
 		printf("remove_fmodifier() - no modifier stack given\n");
 		MEM_freeN(fcm);
-		return 0;
+		return false;
 	}
 }
 
@@ -1264,11 +1262,11 @@ bool list_has_suitable_fmodifier(ListBase *modifiers, int mtype, short acttype)
 		
 	/* sanity checks */
 	if (ELEM(NULL, modifiers, modifiers->first))
-		return 0;
+		return false;
 		
 	/* find the first mdifier fitting these criteria */
 	for (fcm = modifiers->first; fcm; fcm = fcm->next) {
-		FModifierTypeInfo *fmi = fmodifier_get_typeinfo(fcm);
+		const FModifierTypeInfo *fmi = fmodifier_get_typeinfo(fcm);
 		short mOk = 1, aOk = 1; /* by default 1, so that when only one test, won't fail */
 		
 		/* check if applicable ones are fullfilled */
@@ -1279,11 +1277,11 @@ bool list_has_suitable_fmodifier(ListBase *modifiers, int mtype, short acttype)
 			
 		/* if both are ok, we've found a hit */
 		if (mOk && aOk)
-			return 1;
+			return true;
 	}
 	
 	/* no matches */
-	return 0;
+	return false;
 }  
 
 /* Evaluation API --------------------------- */
@@ -1298,7 +1296,7 @@ FModifierStackStorage *evaluate_fmodifiers_storage_new(ListBase *modifiers)
 	}
 
 	for (fcm = modifiers->last; fcm; fcm = fcm->prev) {
-		FModifierTypeInfo *fmi = fmodifier_get_typeinfo(fcm);
+		const FModifierTypeInfo *fmi = fmodifier_get_typeinfo(fcm);
 
 		if (fmi == NULL) {
 			continue;
@@ -1391,6 +1389,8 @@ static float eval_fmodifier_influence(FModifier *fcm, float evaltime)
  *	  working on the 'global' result of the modified curve, not some localised segment,
  *	  so nevaltime gets set to whatever the last time-modifying modifier likes...
  *	- we start from the end of the stack, as only the last one matters for now
+ *
+ * Note: *fcu might be NULL
  */
 float evaluate_time_fmodifiers(FModifierStackStorage *storage, ListBase *modifiers,
                                FCurve *fcu, float cvalue, float evaltime)
@@ -1400,7 +1400,10 @@ float evaluate_time_fmodifiers(FModifierStackStorage *storage, ListBase *modifie
 	/* sanity checks */
 	if (ELEM(NULL, modifiers, modifiers->last))
 		return evaltime;
-		
+
+	if (fcu && fcu->flag & FCURVE_MOD_OFF)
+		return evaltime;
+
 	/* Starting from the end of the stack, calculate the time effects of various stacked modifiers 
 	 * on the time the F-Curve should be evaluated at. 
 	 *
@@ -1412,7 +1415,7 @@ float evaluate_time_fmodifiers(FModifierStackStorage *storage, ListBase *modifie
 	 * (such as multiple 'stepped' modifiers in sequence, causing different stepping rates)
 	 */
 	for (fcm = modifiers->last; fcm; fcm = fcm->prev) {
-		FModifierTypeInfo *fmi = fmodifier_get_typeinfo(fcm);
+		const FModifierTypeInfo *fmi = fmodifier_get_typeinfo(fcm);
 		
 		if (fmi == NULL) 
 			continue;
@@ -1457,10 +1460,13 @@ void evaluate_value_fmodifiers(FModifierStackStorage *storage, ListBase *modifie
 	/* sanity checks */
 	if (ELEM(NULL, modifiers, modifiers->first))
 		return;
+
+	if (fcu->flag & FCURVE_MOD_OFF)
+		return;
 	
 	/* evaluate modifiers */
 	for (fcm = modifiers->first; fcm; fcm = fcm->next) {
-		FModifierTypeInfo *fmi = fmodifier_get_typeinfo(fcm);
+		const FModifierTypeInfo *fmi = fmodifier_get_typeinfo(fcm);
 		
 		if (fmi == NULL) 
 			continue;
